<?php

namespace Drupal\twig_blocks\Twig;

use Drupal\Core\Render\RendererInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\twig_blocks\View\BlockViewBuilder;
use Twig\Extension\AbstractExtension;
use Twig\TwigFunction;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Adds twig extension to render a block.
 */
class RenderBlock extends AbstractExtension {

  /**
   * The renderer.
   *
   * @var \Drupal\Core\Render\RendererInterface
   */
  protected $renderer;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The block view builder.
   *
   * @var \Drupal\twig_blocks\View\BlockViewBuilder
   */
  protected $blockViewBuiler;

  /**
   * Constructor method.
   *
   * @param \Drupal\Core\Render\RendererInterface $renderer
   *   The renderer.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   * @param \Drupal\twig_blocks\View\BlockViewBuilder $blockViewBuilder
   *   The block view builder.
   */
  public function __construct(RendererInterface $renderer, EntityTypeManagerInterface $entity_type_manager, BlockViewBuilder $blockViewBuilder) {
    $this->renderer = $renderer;
    $this->entityTypeManager = $entity_type_manager;
    $this->blockViewBuiler = $blockViewBuilder;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('renderer'),
      $container->get('entity_type.manager'),
      $container->get('twig_blocks.block_view_builder')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getName() {
    return 'render_block';
  }

  /**
   * {@inheritdoc}
   */
  public function getFunctions() {
    return [
      new TwigFunction('render_block', [$this, 'renderBlock'], ['is_safe' => ['html']]),
    ];
  }

  /**
   * Provides function to programmatically rendering a block.
   *
   * @param string $block_id
   *   The machine id of the block to render.
   * @param array $configuration
   *   The configuration for the block.
   * @param bool $wrapper
   *   The wrapper for the block.
   *
   * @return array
   *   Returns render array of block.
   *
   * @throws \Exception
   */
  public function renderBlock($block_id, array $configuration = [], bool $wrapper = TRUE) {
    return $this->blockViewBuiler->build($block_id, $configuration, $wrapper);
  }

}
